aboutsummaryrefslogtreecommitdiffhomepage
path: root/src/runtime/debug
diff options
context:
space:
mode:
authorLaurent Demailly <[email protected]>2024-07-20 06:26:08 -0700
committerGitHub <[email protected]>2024-07-20 15:26:08 +0200
commit89340f82dc65ec6b1adec75efec2dafe78368a40 (patch)
treea281517348502dd5d310e7c3640e0f1f9acc629c /src/runtime/debug
parent824bf244453bdb0502fa30de70c799d666665480 (diff)
downloadtinygo-89340f82dc65ec6b1adec75efec2dafe78368a40.tar.gz
tinygo-89340f82dc65ec6b1adec75efec2dafe78368a40.zip
Allows compilation of code using debug.BuildInfo and show correct tinygo version (#4343)
debug: Allows compilation of code using debug.BuildInfo and show correct tinygo version
Diffstat (limited to 'src/runtime/debug')
-rw-r--r--src/runtime/debug/debug.go81
1 files changed, 75 insertions, 6 deletions
diff --git a/src/runtime/debug/debug.go b/src/runtime/debug/debug.go
index 763117313..38e6ab763 100644
--- a/src/runtime/debug/debug.go
+++ b/src/runtime/debug/debug.go
@@ -1,6 +1,17 @@
-// Package debug is a dummy package that is not yet implemented.
+// Portions copyright 2009 The Go Authors. All rights reserved.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file.
+
+// Package debug is a very partially implemented package to allow compilation.
package debug
+import (
+ "fmt"
+ "runtime"
+ "strconv"
+ "strings"
+)
+
// SetMaxStack sets the maximum amount of memory that can be used by a single
// goroutine stack.
//
@@ -27,16 +38,17 @@ func Stack() []byte {
//
// Not implemented.
func ReadBuildInfo() (info *BuildInfo, ok bool) {
- return nil, false
+ return &BuildInfo{GoVersion: runtime.Compiler + runtime.Version()}, true
}
// BuildInfo represents the build information read from
// the running binary.
type BuildInfo struct {
- Path string // The main package path
- Main Module // The module containing the main package
- Deps []*Module // Module dependencies
- Settings []BuildSetting
+ GoVersion string // version of the Go toolchain that built the binary, e.g. "go1.19.2"
+ Path string // The main package path
+ Main Module // The module containing the main package
+ Deps []*Module // Module dependencies
+ Settings []BuildSetting
}
type BuildSetting struct {
@@ -58,3 +70,60 @@ type Module struct {
func SetGCPercent(n int) int {
return n
}
+
+// Start of stolen from big go. TODO: import/reuse without copy pasta.
+
+// quoteKey reports whether key is required to be quoted.
+func quoteKey(key string) bool {
+ return len(key) == 0 || strings.ContainsAny(key, "= \t\r\n\"`")
+}
+
+// quoteValue reports whether value is required to be quoted.
+func quoteValue(value string) bool {
+ return strings.ContainsAny(value, " \t\r\n\"`")
+}
+
+func (bi *BuildInfo) String() string {
+ buf := new(strings.Builder)
+ if bi.GoVersion != "" {
+ fmt.Fprintf(buf, "go\t%s\n", bi.GoVersion)
+ }
+ if bi.Path != "" {
+ fmt.Fprintf(buf, "path\t%s\n", bi.Path)
+ }
+ var formatMod func(string, Module)
+ formatMod = func(word string, m Module) {
+ buf.WriteString(word)
+ buf.WriteByte('\t')
+ buf.WriteString(m.Path)
+ buf.WriteByte('\t')
+ buf.WriteString(m.Version)
+ if m.Replace == nil {
+ buf.WriteByte('\t')
+ buf.WriteString(m.Sum)
+ } else {
+ buf.WriteByte('\n')
+ formatMod("=>", *m.Replace)
+ }
+ buf.WriteByte('\n')
+ }
+ if bi.Main != (Module{}) {
+ formatMod("mod", bi.Main)
+ }
+ for _, dep := range bi.Deps {
+ formatMod("dep", *dep)
+ }
+ for _, s := range bi.Settings {
+ key := s.Key
+ if quoteKey(key) {
+ key = strconv.Quote(key)
+ }
+ value := s.Value
+ if quoteValue(value) {
+ value = strconv.Quote(value)
+ }
+ fmt.Fprintf(buf, "build\t%s=%s\n", key, value)
+ }
+
+ return buf.String()
+}